
       subroutine fermion_fill(mup,mdown,centers,rotmat,vp,pos)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          real*8 centers(ndim,npair),pos(ndim,nelec)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
 
          integer i,j
          real*8 ob(npair)
 
          do i=1,nup
             call orbital(pos(1,i),centers,rotmat,vp,ob)
             do j=1,nup
                mup(i,j) = ob(j)
             enddo
          enddo


          do i=1,ndown
             call orbital(pos(1,nup+i),centers,rotmat,vp,ob)
             do j=1,ndown
                mdown(i,j) = ob(j)
             enddo
          enddo

       end


       subroutine fermion_det(mup,mdown,det)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          real*8 det

          integer info,i
          integer ipiv(nspinmax)
          real*8 detup,detdown
          real*8 work(nspinmax)

          call dgetrf(nup,nup,mup,nup,ipiv,info)
c          if (info.ne.0) print*,'up,info from dgetrf: ',info
          if (info.ne.0) lapack_info = info

          detup = 1.0d0
          do i=1,nup
             if (i.ne.ipiv(i)) detup=-detup
             detup = detup * mup(i,i)
          enddo

          call dgetri(nup,mup,nup,ipiv,work,nspinmax,info)
c          if (info.ne.0) print*,'up,info from dgetri: ',info
          if (info.ne.0) lapack_info = info

          call dgetrf(ndown,ndown,mdown,ndown,ipiv,info)
c          if (info.ne.0) print*,'down,info from dgetrf: ',info
          if (info.ne.0) lapack_info = info

          detdown = 1.0d0
          do i=1,ndown
             if (i.ne.ipiv(i)) detdown=-detdown
             detdown = detdown * mdown(i,i)
          enddo

          call dgetri(ndown,mdown,ndown,ipiv,work,nspinmax,info)
c          if (info.ne.0) print*,'down,info from dgetri: ',info
          if (info.ne.0) lapack_info = info

          det = detup*detdown

       end

       subroutine fermion_det_only(mup,mdown,det)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          real*8 det

          integer info,i
          integer ipiv(nspinmax)
          real*8 detup,detdown
          real*8 work(nspinmax)

          call dgetrf(nup,nup,mup,nup,ipiv,info)
c          if (info.ne.0) print*,'up,info from dgetrf: ',info
          if (info.ne.0) lapack_info = info

          detup = 1.0d0
          do i=1,nup
             if (i.ne.ipiv(i)) detup=-detup
             detup = detup * mup(i,i)
          enddo

          call dgetrf(ndown,ndown,mdown,ndown,ipiv,info)
c          if (info.ne.0) print*,'down,info from dgetrf: ',info
          if (info.ne.0) lapack_info = info

          detdown = 1.0d0
          do i=1,ndown
             if (i.ne.ipiv(i)) detdown=-detdown
             detdown = detdown * mdown(i,i)
          enddo

          det = detup*detdown

       end


       subroutine fermion_det_ratio(mup,mdown,indx,r,ratio,centers,
     >              rotmat,vp)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          integer indx
          real*8 r(ndim),ratio
          real*8 centers(ndim,npair)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)

          integer ix,i
          real*8 dot,ob(npair)


          call orbital(r,centers,rotmat,vp,ob)
          if (indx.le.nup)  then
             dot = 0.d0
             do i=1,nup
                dot = dot + ob(i)*mup(i,indx)
             enddo
          else
             ix = indx-nup
             dot = 0.d0
             do i=1,ndown
                dot = dot + ob(i)*mdown(i,ix)
             enddo
          endif

          ratio = dot
           
       end

       subroutine fermion_update(mup,mdown,indx,r,centers,rotmat,vp)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          integer indx
          real*8 r
          real*8 centers(ndim,npair)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
        
          integer ix,i
          real*8 ob(npair)

          call orbital(r,centers,rotmat,vp,ob)
          if (indx.le.nup) then
             do i=1,nup
                mup(indx,i) = ob(i)
             enddo
          else
             ix = indx-nup
             do i=1,nup
                mdown(ix,i) = ob(i)
             enddo
          endif
     
       end

       subroutine fermion_inv_update(mup,mdown,indx,r,centers,rotmat,vp,
     >                               q)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          integer indx
          real*8 r(ndim),q
          real*8 centers(ndim,npair)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
    
          real*8 ob(npair),tmp(nspinmax)
          real*8 sum
          integer i,j,ik

          call orbital(r,centers,rotmat,vp,ob)

          if (indx.le.nup) then
             do i=1,nup
                sum = 0.d0
                do j=1,nup
                   sum = sum + mup(j,i)*ob(j)
                enddo
                tmp(i) = sum
             enddo
             do i=1,nup
                if (indx.ne.i) then
                   do j=1,nup
                      mup(j,i) = mup(j,i)-mup(j,indx)*tmp(i)/q
                   enddo
                endif
             enddo
             do j=1,nup
                mup(j,indx) = mup(j,indx)/q
             enddo
 
          else
             ik = indx-nup
             do i=1,ndown
                sum = 0.d0
                do j=1,ndown
                   sum = sum + mdown(j,i)*ob(j)
                enddo
                tmp(i) = sum
             enddo
             do i=1,ndown
                if (i.ne.ik) then
                   do j=1,ndown
                      mdown(j,i) = mdown(j,i)-mdown(j,ik)*tmp(i)/q
                   enddo
                endif
             enddo
             do j=1,ndown
                mdown(j,ik) = mdown(j,ik)/q
             enddo
              
          endif
        

       end

       subroutine fermion_grad(mup,mdown,pos,centers,rotmat,vp,
     >                         gd,sum_gdd)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          real*8 centers(ndim,npair),pos(ndim,nelec)
          real*8 gd(ndim,nelec),sum_gdd
          real*8 rotmat(ndim,ndim,npair),vp(nvp)

          integer i,j,k,ix
          real*8 ob(nspinmax),ob_d(ndim,nspinmax),ob_dd(nspinmax)
          real*8 t(ndim)

          sum_gdd = 0.d0
          do i=1,nup
             call orbital_d(pos(1,i),centers,rotmat,vp,ob,ob_d,ob_dd)
             do j=1,ndim
               t(j) = 0.d0
             enddo
             do k=1,nup
                do j=1,ndim
                  t(j) = t(j) + mup(k,i) * ob_d(j,k)
                 enddo
             enddo
             do j=1,ndim
                gd(j,i) = t(j)
             enddo
 
             do k=1,nup
                sum_gdd = sum_gdd + mup(k,i)*ob_dd(k)
             enddo
          enddo

          do i=1,ndown
             ix = i + nup
             call orbital_d(pos(1,ix),centers,rotmat,vp,ob,ob_d,ob_dd)
             do j=1,ndim
               t(j) = 0.d0
             enddo
             do k=1,ndown
                do j=1,ndim
                  t(j) = t(j) + mdown(k,i) * ob_d(j,k)
                 enddo
             enddo
             do j=1,ndim
                gd(j,ix) = t(j)
             enddo
            
             do k=1,ndown
                sum_gdd = sum_gdd + mdown(k,i)*ob_dd(k)
             enddo
          enddo

       end

       subroutine fermion_grad_one(mup,mdown,centers,rotmat,vp,
     >                         gd,sum_gdd,indx,xt)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown)
          real*8 centers(ndim,npair)
          real*8 gd(ndim),sum_gdd
          real*8 rotmat(ndim,ndim,npair),vp(nvp),xt(ndim)
          integer indx

          integer i,j,k,ix
          real*8 ob(nspinmax),ob_d(ndim,nspinmax),ob_dd(nspinmax)
          real*8 t(ndim)

         
          sum_gdd = 0.d0

          if (indx.le.nup) then
             call orbital_d(xt,centers,rotmat,vp,ob,ob_d,ob_dd)
             do j=1,ndim
               t(j) = 0.d0
             enddo
             do k=1,nup
                do j=1,ndim
                  t(j) = t(j) + mup(k,indx) * ob_d(j,k)
                 enddo
             enddo
             do j=1,ndim
                gd(j) = t(j)
             enddo

             do k=1,nup
                sum_gdd = sum_gdd + mup(k,indx)*ob_dd(k)
             enddo

          else

             ix = indx-nup
             call orbital_d(xt,centers,rotmat,vp,ob,ob_d,ob_dd)
             do j=1,ndim
               t(j) = 0.d0
             enddo
             do k=1,ndown
                do j=1,ndim
                  t(j) = t(j) + mdown(k,ix) * ob_d(j,k)
                 enddo
             enddo
             do j=1,ndim
                gd(j) = t(j)
             enddo
         
             do k=1,ndown
                sum_gdd = sum_gdd + mdown(k,ix)*ob_dd(k)
             enddo
          endif

       end

       subroutine fermion_vp_deriv(mup,mdown,pos,centers,rotmat,vp,grad)
          implicit none
          include 'param.h'
          real*8 mup(nup,nup),mdown(ndown,ndown),pos(ndim,nnuclei)
          real*8 centers(ndim,npair),rotmat(ndim,ndim,npair)
          real*8 grad(npair*nvp_per),vp(nvp)

          integer i,l,j,ib
          real*8 work(npair,nvp_per),dot

          do i=1,nup
             ib = nvp_per*(i-1)
             call orbital_vp(nup,i,pos(1,1),centers(1,i),rotmat,vp,work)
             do l=1,3
                dot = 0.d0
                do j=1,nup
                   dot = dot + mup(i,j)*work(j,l)
                enddo
                grad(ib+l) = dot
             enddo
          enddo
          do i=1,ndown
             ib = nvp_per*(i-1)
             call orbital_vp(ndown,i,pos(1,nup+1),centers(1,i),rotmat,
     >                       vp,work)
             do l=1,3
                dot = 0.d0
                do j=1,nup
                   dot = dot + mdown(i,j)*work(j,l)
                enddo
                grad(ib+l) = grad(ib+l) + dot
             enddo
          enddo

    

         
       end

