
       subroutine ee_jastrow(dist,vp,sum)
          implicit none
          include 'param.h'
          real*8 dist(nelec-1),vp(nvp)
          real*8 sum

          real*8 rm,v,y,f,z,ch(n_chby_max)
          integer i,in
          real*8 cusp
          parameter (cusp = -0.5d0)

          rm =  .5d0*box_len
          rm = min(rm,vp(1))

          sum=0.d0
          do i=1,nelec-1
             y = dist(i)/rm 
             if (y.lt.1.d0) then
                v = cusp*rm*(y - y*y + y**3/3.d0)
                f = y*y*(y-1.d0)**3
                z = 2*y-1.d0
                ch(1) = 1.d0
                ch(2) = z
                ch(3) = 2*z**2 - 1.d0
                ch(4) = 4*z**3 - 3*z
                ch(5) = 8*z**4 - 8*z**2 + 1.d0
                do in=1,n_chby
                   v = v + f*ch(in)*vp(1+in)
                enddo
             else
                v = cusp*rm/3.d0
             endif
             sum = sum + v
          enddo

       end

       subroutine en_jastrow(dist,vp,sum)
          implicit none
          include 'param.h'
          real*8 dist(nnuclei),vp(nvp)
          real*8 sum

          real*8 rm,v,y,f,z,ch(n_chby_max)
          integer i,in
          real*8 cusp
          parameter (cusp = 1.d0)

          rm =  .5d0*box_len
          rm = min(rm,vp(ien))

          sum=0.d0
          do i=1,nnuclei
             y = dist(i)/rm 
             if (y.lt.1.d0) then
                v = cusp*rm*(y - y*y + y**3/3.d0)
                f = y*y*(y-1.d0)**3
                z = 2*y-1.d0
                ch(1) = 1.d0
                ch(2) = z
                ch(3) = 2*z**2 - 1.d0
                ch(4) = 4*z**3 - 3*z
                ch(5) = 8*z**4 - 8*z**2 + 1.d0
                do in=1,n_chby
                   v = v + f*ch(in)*vp(ien+in)
                enddo
             else
                v = cusp*rm/3.d0
             endif
             sum = sum + v
          enddo

       end

       subroutine ee_jastrow_d(dist,vp,sum,sum_delsqu,gradu)
          implicit none
          include 'param.h'
          real*8 dist(nelec-1),vp(nvp)
          real*8 sum,sum_delsqu
          real*8 gradu(nelec-1)

          real*8 rm,v,y,f,z,df,ddf
          real*8 ch(n_chby_max),dch(n_chby_max),ddch(n_chby_max)
          real*8 dv,ddv
          integer i,in
          real*8 cusp
          parameter (cusp = -0.5d0)

          rm =  .5d0*box_len
          rm = min(rm,vp(1))

          sum=0.d0
          sum_delsqu = 0.d0
          do i=1,nelec-1
             y = dist(i)/rm 
             if (y.lt.1.d0) then
                v = cusp*rm*(y - y*y + y**3/3.d0)
                dv = cusp*(1.d0 -2*y + y**2)
                ddv = 2*cusp*(-1.d0+y)/rm
                f = y*y*(y-1.d0)**3
                df = (2*y*(y-1.d0)**3 + 3*y*y*(y-1.d0)**2)/rm
                ddf = (2*(y-1.d0)**3 + 12*y*(y-1.d0)**2 + 
     >                 6*y*y*(y-1.d0))/rm**2
                z = 2*y-1.d0

                ch(1) = 1.d0
                dch(1) = 0.d0
                ddch(1) = 0.d0

                ch(2) = z
                dch(2) = 2/rm
                ddch(2) = 0.d0

                ch(3) = 2*z**2 - 1.d0
                dch(3) = 8*z/rm
                ddch(3) = 16/rm**2

                ch(4) = 4*z**3 - 3*z
                dch(4) = (24*z**2 - 6.d0)/rm
                ddch(4) = 96*z/rm**2

                ch(5) = 8*z**4 - 8*z**2 + 1.d0
                dch(5) = (64*z**3 - 32*z)/rm
                ddch(5) = (384*z**2 - 64.d0)/rm**2
     
                do in=1,n_chby
                   v = v + vp(1+in)*f*ch(in)
                   dv = dv + vp(1+in) *(f*dch(in) + df*ch(in))
                   ddv = ddv + vp(1+in) *(f*ddch(in) + ddf*ch(in) + 
     >                  2*df*dch(in))
                enddo
             else
                v = cusp*rm/3.d0
                dv = 0.d0
                ddv = 0.d0
             endif
             gradu(i) =  dv
             sum = sum + v
             sum_delsqu = sum_delsqu + ddv
          enddo

       end

       subroutine en_jastrow_d(dist,vp,sum,sum_delsqu,gradu)
          implicit none
          include 'param.h'
          real*8 dist(nnuclei),vp(nvp)
          real*8 sum,sum_delsqu
          real*8 gradu(nnuclei)

          real*8 rm,v,y,f,z,df,ddf
          real*8 ch(n_chby_max),dch(n_chby_max),ddch(n_chby_max)
          real*8 dv,ddv
          integer i,in
          real*8 cusp
          parameter (cusp = 1.0d0)

          rm =  .5d0*box_len
          rm = min(rm,vp(ien))

          sum=0.d0
          sum_delsqu = 0.d0
          do i=1,nnuclei
             y = dist(i)/rm 
             if (y.lt.1.d0) then
                v = cusp*rm*(y - y*y + y**3/3.d0)
                dv = cusp*(1.d0 -2*y + y**2)
                ddv = -2*cusp*(1.d0-y)/rm
                f = y*y*(y-1.d0)**3
                df = (2*y*(y-1.d0)**3 + 3*y*y*(y-1.d0)**2)/rm
                ddf = (2*(y-1.d0)**3 + 12*y*(y-1.d0)**2 + 
     >                 6*y*y*(y-1.d0))/rm**2

                z = 2*y-1.d0
                ch(1) = 1.d0
                dch(1) = 0.d0
                ddch(1) = 0.d0

                ch(2) = z
                dch(2) = 2/rm
                ddch(2) = 0.d0

                ch(3) = 2*z**2 - 1.d0
                dch(3) = 8*z/rm
                ddch(3) = 16/rm**2

                ch(4) = 4*z**3 - 3*z
                dch(4) = (24*z**2 - 6.d0)/rm
                ddch(4) = 96*z/rm**2

                ch(5) = 8*z**4 - 8*z**2 + 1.d0
                dch(5) = (64*z**3 - 32*z)/rm
                ddch(5) = (384*z**2 - 64.d0)/rm**2
     
                do in=1,n_chby
                   v = v + vp(ien+in)*f*ch(in)
                   dv = dv + vp(ien+in) *(f*dch(in) + df*ch(in))
                   ddv = ddv + vp(ien+in) *(f*ddch(in) + ddf*ch(in) + 
     >                  2*df*dch(in))
                enddo
             else
                v = cusp*rm/3.d0
                dv = 0.d0
                ddv = 0.d0
             endif
             gradu(i) =  dv
             sum = sum + v
             sum_delsqu = sum_delsqu + ddv
          enddo

       end

       subroutine ee_jastrow_d_vp(dist,vp,sum,sum_delsqu,gradu,vp_deriv)
          implicit none
          include 'param.h'
          real*8 dist(nelec-1),vp(nvp)
          real*8 sum,sum_delsqu
          real*8 gradu(nelec-1)
          real*8 vp_deriv(nvp_ee)

          real*8 rm,v,y,f,z,df,ddf
          real*8 ch(n_chby_max),dch(n_chby_max),ddch(n_chby_max)
          real*8 dv,ddv
          integer i,in
          real*8 cusp
          parameter (cusp = -0.5d0)

          rm =  .5d0*box_len
          rm = min(rm,vp(1))

          sum=0.d0
          sum_delsqu = 0.d0
          do i=1,nelec-1
             y = dist(i)/rm 
             if (y.lt.1.d0) then
                v = cusp*rm*(y - y*y + y**3/3.d0)
                dv = cusp*(1.d0 -2*y + y**2)
                ddv = -2*cusp*(1.d0-y)/rm
                f = y*y*(y-1.d0)**3
                df = (2*y*(y-1.d0)**3 + 3*y*y*(y-1.d0)**2)/rm
                ddf = (2*(y-1.d0)**3 + 12*y*(y-1.d0)**2 + 
     >                 6*y*y*(y-1.d0))/rm**2
                z = 2*y-1.d0
                ch(1) = 1.d0
                dch(1) = 0.d0
                ddch(1) = 0.d0

                ch(2) = z
                dch(2) = 2/rm
                ddch(2) = 0.d0

                ch(3) = 2*z**2 - 1.d0
                dch(3) = 8*z/rm
                ddch(3) = 16/rm**2

                ch(4) = 4*z**3 - 3*z
                dch(4) = (24*z**2 - 6.d0)/rm
                ddch(4) = 96*z/rm**2

                ch(5) = 8*z**4 - 8*z**2 + 1.d0
                dch(5) = (64*z**3 - 32*z)/rm
                ddch(5) = (384*z**2 - 64.d0)/rm**2
     
                vp_deriv(1) = vp_deriv(1) - cusp*(y**2 - 2*y**3/3)/2.d0
                do in=1,n_chby
                   v = v + f*vp(1+in)*ch(in)
                   dv = dv + vp(1+in) *(f*dch(in) + df*ch(in))
                   ddv = ddv + vp(1+in) *(f*ddch(in) + ddf*ch(in) + 
     >                  2*df*dch(in))
                   vp_deriv(in+1) = vp_deriv(in+1) - f*ch(in)/2.d0
                enddo
             else
                v = cusp*rm/3.d0
                dv = 0.d0
                ddv = 0.d0
                vp_deriv(1) = vp_deriv(1) - cusp/6.d0
             endif
             sum = sum + v
             gradu(i) =  dv
             sum_delsqu = sum_delsqu + ddv
          enddo

       end

       subroutine en_jastrow_d_vp(dist,vp,sum,sum_delsqu,gradu,vp_deriv)
          implicit none
          include 'param.h'
          real*8 dist(nnuclei),vp(nvp)
          real*8 sum,sum_delsqu
          real*8 gradu(nnuclei)
          real*8 vp_deriv(nvp_en)

          real*8 rm,v,y,f,z,df,ddf
          real*8 ch(n_chby_max),dch(n_chby_max),ddch(n_chby_max)
          real*8 dv,ddv
          integer i,in
          real*8 cusp
          parameter (cusp = 1.0d0)

          rm =  .5d0*box_len
          rm = min(rm,vp(ien))

          sum=0.d0
          sum_delsqu = 0.d0
          do i=1,nnuclei
             y = dist(i)/rm 
             if (y.lt.1.d0) then
                v = cusp*rm*(y - y*y + y**3/3.d0)
                dv = cusp*(1.d0 -2*y + y**2)
                ddv = -2*cusp*(1.d0-y)/rm
                f = y*y*(y-1.d0)**3
                df = (2*y*(y-1.d0)**3 + 3*y*y*(y-1.d0)**2)/rm
                ddf = (2*(y-1.d0)**3 + 12*y*(y-1.d0)**2 + 
     >                 6*y*y*(y-1.d0))/rm**2
                z = 2*y-1.d0
                ch(1) = 1.d0
                dch(1) = 0.d0
                ddch(1) = 0.d0

                ch(2) = z
                dch(2) = 2/rm
                ddch(2) = 0.d0

                ch(3) = 2*z**2 - 1.d0
                dch(3) = 8*z/rm
                ddch(3) = 16/rm**2

                ch(4) = 4*z**3 - 3*z
                dch(4) = (24*z**2 - 6.d0)/rm
                ddch(4) = 96*z/rm**2

                ch(5) = 8*z**4 - 8*z**2 + 1.d0
                dch(5) = (64*z**3 - 32*z)/rm
                ddch(5) = (384*z**2 - 64.d0)/rm**2
     
                vp_deriv(1) = vp_deriv(1) - cusp*(y**2 - 2*y**3/3)
                do in=1,n_chby
                   v = v + f*vp(ien+in)*ch(in)
                   dv = dv + vp(ien+in) *(f*dch(in) + df*ch(in))
                   ddv = ddv + vp(ien+in) *(f*ddch(in) + ddf*ch(in) + 
     >                  2*df*dch(in))
                   vp_deriv(in+1) = vp_deriv(in+1) - f*ch(in)
                enddo
             else
                v = cusp*rm/3.d0
                dv = 0.d0
                ddv = 0.d0
                vp_deriv(1) = vp_deriv(1) - cusp/3.d0
             endif
             sum = sum + v
             gradu(i) =  dv
             sum_delsqu = sum_delsqu + ddv
          enddo

       end
