
c Main VMC program
      implicit none
      include 'param.h'

c storage created here
      real*8 epos(ndim,nelec),npos(ndim,nnuclei)
      real*8 centers(ndim,npair),rotmat(ndim,ndim,npair)
      real*8 mup(nup,nup),mdown(ndown,ndown)
      real*8 vp(nvp),tmp(nvp)


c local vars
      integer ns,j,np,nac,nacc,nb,i
      real*8 det,en,nuc_pot,pt,dr,ar,sb
      real*8 sum_en,ke,pe,lpsi,ssq_en,var
      real*8 xpos,ypos
      real*8 rng
      external rng
      real*8 tv(nnuclei),tv3(ndim,nnuclei),b(nvp),scale(nvp)
      real*8 beta,wk,sca
c simulation variables
      integer nsteps,iseed,nblocks,nerrblocks,itmax,vmc_nblocks
      character*40 vp_file,pos_file,vp_out_file,infile,curr_file
      logical do_min,do_grad,do_energy
     

      call parallel_init()
      do_min = .true.
      do_energy = .true.
      do_grad = .false.
      iseed = 101
      call easy_init_rng(iseed)
      if (root) then
         call getarg(1,infile)
         curr_file = infile
         open(5,file=infile,err=911)
      endif
      if (root) print*,'nproc = ',nproc


c      print*,'nsteps'
c      read*,nsteps
      call read_integer_par(nsteps,'nsteps')
c      print*,'nblocks'
c      read*,nblocks
      call read_integer_par(nblocks,'nblocks')
c      print*,'vmc nblocks'
c      read*,vmc_nblocks
      call read_integer_par(vmc_nblocks,'vmc_nblocks')
      nerrblocks = 10
      call read_integer_par(nerrblocks,'nerrblocks')

c set vp's and sim parameters

      pi = 4.d0*atan(1.d0)
c      rs = 3.d0
      rs = 1.8d0
      density = 3.d0/(4.d0*pi*rs**3)
      box_len = ( (2*npair)/density)**(1.d0/3.d0)
      recip_box_len = 1.d0/box_len

      if (root) then
         print*,'box_len = ',box_len

         print*,'nvp = ',nvp

         print*,'vp file'
         read(5,'(a40)') vp_file
         print*,'vp out file'
         read(5,'(a40)') vp_out_file
         open(15,file=vp_file,status='OLD',err=911)
         do i=1,nvp
            read(15,*) vp(i)
         enddo
         close(15)
      endif
      call bcast_array(vp,nvp)


c      print*,'scale factor'
c      read*,sca
      call read_double_par(sca,'scale factor')
      do i=1,nvp
         scale(i) = sca
      enddo
c      scale(1) = .4d0
c      scale(ien) = .4d0
      
      itmax = 2
c      print*,'itmax'
c      read*,itmax
      call read_integer_par(itmax,'itmax')

      delta = 1.d0
c      print*,'delta'
c      read*,delta
      call read_double_par(delta,'delta')


 
c set up intial positions

c      xpos = 1.4d0
c      ypos = 1.4d0
c      do i=1,npair
c         do j=1,ndim
c           npos(j,2*i-1) = 0.d0
c           npos(j,2*i) = 0.d0
c         enddo
c      enddo
c      do j=1,npair
c         npos(1,2*j) = xpos
c         npos(2,2*j-1) = (j-1)*ypos
c         npos(2,2*j) = (j-1)*ypos
c      enddo
      if (root) then
         print*,'nuc pos file'
         read(5,'(a40)') pos_file
         open(10,file=pos_file,status='OLD',err=911)
         do i=1,npair
             read(10,*) (npos(j,2*i-1),j=1,ndim)
             read(10,*) (npos(j,2*i),j=1,ndim)
         enddo
         close(10)
      endif
      call bcast_array(npos,2*npair*ndim)
      

      call set_centers(npos,centers,rotmat)
 
c      print*,'center',centers(1,1),centers(2,1),centers(3,1)

      sb = 2.d0
      do np=1,npair
         do j=1,ndim
c            epos(j,np) = box_len*(rng()-.5d0)
             epos(j,np) = centers(j,np) + sb*(rng()-.5d0)
             epos(j,np+npair) = centers(j,np) + sb*(rng()-.5d0)

c            epos(j,np) = box_len*rng()
         enddo
      enddo

      nuc_pot = 0.d0
      do j=1,nnuclei
         call nn_dist(j,npos,npos(1,j),tv,tv3)
         call nn_potential(tv,pt)
         nuc_pot = nuc_pot + 0.5d0*pt
      enddo
      if (root) print*,'Nuclear potential ',nuc_pot

      call total_nuclear_potential(npos,nuc_pot)
      if (root) print*,'Alt Nuclear potential ',nuc_pot
c set up fermion matrices

      call fermion_fill(mup,mdown,centers,rotmat,vp,epos)
      call fermion_det(mup,mdown,det)
      if (root) print*,'det',det

      call local_energy(npos,epos,mup,mdown,centers,rotmat,vp,en,ke,pe,
     >                  lpsi)
      if (root) print*,'local e',en
      dr = 2.0d-4
c      call wf_test(npos,epos,mup,mdown,centers,rotmat,vp,dr)

      if (do_min) then
         call optimize(nsteps,nblocks,nerrblocks,itmax,
     >                 npos,epos,mup,mdown,centers,rotmat,vp,scale)
      endif

      sum_en = 0.d0
      ssq_en = 0.d0
      nacc = 0
      if (do_energy) then
      do nb=1,vmc_nblocks
         do ns=1,nsteps
            call metropolis(mup,mdown,npos,epos,centers,rotmat,vp,det,
     >                      nac)
            nacc = nacc + nac
         enddo
         call local_energy(npos,epos,mup,mdown,centers,rotmat,vp,en,
     >                     ke,pe,lpsi)
         call sum_reduce(en)
         en = en/nproc
         sum_en = sum_en + en
         ssq_en = ssq_en + en**2
         if (root) write(40,'(2e20.10)') en+nuc_pot,en
c         write(41,'(6e14.5)') epos(1,1),epos(2,1),epos(3,1),
c     >       epos(1,2),epos(2,2),epos(3,2)
      enddo
      endif

      if (do_grad) then
      dr = 1.0d-3
      call estimate_gradient(npos,epos,mup,mdown,centers,rotmat,vp,dr,
     >                       nsteps,nblocks,en,b)
c      call compute_gradient(npos,epos,mup,mdown,centers,rotmat,vp,det,
c     >                       dr,en,b)
      call sum_reduce_array(vp,tmp,nvp)
      if (root) then
         do i=1,nvp
            print*,'b:',i,b(i)
         enddo
      endif
      endif
     


c      print*,'ave energy = ',sum_en/nblocks + nuc_pot
      en = sum_en/vmc_nblocks
      var = ssq_en/vmc_nblocks-en*en
      if (root) then
         print*,'ave energy = ',en +nuc_pot,sqrt(var/(vmc_nblocks-1))
         open(15,file=vp_out_file,err=911)
         do i=1,nvp
            write(15,*) vp(i)
         enddo
         close(15)
      endif
  

      ar = real(nacc)/real(nsteps*nelec*vmc_nblocks)
      call sum_reduce(ar)
      if (root) then
         print*,'acceptance ratio',ar/nproc
      endif


      call parallel_end()
      stop


911   continue
      if (root) print*,'file not found'
      call parallel_end()
      stop

      end
      
