
c  subroutines for generating configurations, and for computing from them
c  using weighted values

      subroutine gen_config(nsteps,nblocks,npos,epos,mup,mdown,centers,
     >                       rotmat,vp)
       implicit none
       include 'param.h' 
       include 'configs.h'
 
       integer nsteps,nblocks
 
       real*8 npos(ndim,nnuclei),epos(ndim,nelec)
       real*8 mup(nup,nup),mdown(ndown,ndown)
       real*8 centers(ndim,npair),rotmat(ndim,ndim,npair),vp(nvp)

 
       integer ns,nb,i,j,k,nac
       real*8 det,en,ke,pe,lpsi
 
       call total_nuclear_potential(npos,config_nuc_pot)
       if (e_trial.eq.0.d0) then
          e_trial = -1.160d0
       endif
       nconfig = 0
       call fermion_fill(mup,mdown,centers,rotmat,vp,epos)
       call fermion_det(mup,mdown,det)
       do nb=1,nblocks
          do ns=1,nsteps
             call metropolis(mup,mdown,npos,epos,centers,rotmat,vp,det,
     >                      nac)
          enddo
          nconfig = nconfig + 1
          if (nconfig.gt.nconfig_max) then
               print*,'ERROR - nconfig too large',nconfig
          endif
          do j=1,nelec
             do i=1,ndim
                config_epos(i,j,nconfig) = epos(i,j) 
             enddo
          enddo
          call local_energy(npos,epos,mup,mdown,centers,rotmat,vp,en,
     >                      ke,pe,lpsi)
          config_lpsi(nconfig) = -lpsi + 2.d0*log(abs(det))
       enddo
       do i=1,nvp
          config_vp(i) = vp(i)
       enddo
       do j=1,npair
          do i=1,ndim
             config_center(i,j) = centers(i,j)
          enddo
       enddo
       do j=1,nnuclei
          do i=1,ndim
             config_npos(i,j) = npos(i,j)
          enddo
       enddo
       do j=1,npair
          do k=1,ndim
             do i=1,ndim
                config_rotmat(i,k,j) = rotmat(i,k,j)
             enddo
          enddo
       enddo

       
 
      end

      subroutine mincalc(n,x,nf,f)
       implicit none
       include 'param.h'
       include 'configs.h'
 
       integer n,nf
       real*8 x(n),f

       integer nc,i,itot
       real*8 mup(nup,nup),mdown(ndown,ndown)
       real*8 det,en,ke,pe,lpsi,neff
       real*8 sum,ssq,var,ave,vsum,err,evar,newvar,sigma
       real*8 wt,wt_sum,wt_ssq,psi,val,limit
       real*8 enlist(nconfig_max)

       real*4 etime,tme1(2),tme2(2),tme,tot1,tot2
       external etime

       do i=1,nvp
          config_vp(i) = x(i)
       enddo
c       config_vp(1) = x(1)
c       config_vp(13) = x(2)
c       config_vp(14) = x(3)
       sum = 0.d0
       ssq = 0.d0
       wt_ssq = 0.d0
       wt_sum = 0.d0
       vsum = 0.d0
       det_time = 0.0
       jas_time = 0.0
       tme = etime(tme1)
       tot1= tme1(1)
       do nc=1,nconfig
c          tme = etime(tme1) 
          call fermion_fill(mup,mdown,config_center,config_rotmat,
     >                      config_vp,config_epos(1,1,nc))
          call fermion_det(mup,mdown,det)

c          tme = etime(tme2) 
c          det_time = det_time + tme2(1) - tme1(1)

          call local_energy(config_npos,config_epos(1,1,nc),mup,mdown,
     >                      config_center,config_rotmat,config_vp,
     >                      en,ke,pe,lpsi)

          en = en + config_nuc_pot
          enlist(nc) = en
          sum = sum + en
          ssq = ssq + en*en
c no restrictions on weights
          psi = -lpsi + 2.d0*log(abs(det))
          wt =  exp(-(psi-config_lpsi(nc)))
          wt_sum = wt_sum + wt
          wt_ssq = wt_ssq + wt*wt
          vsum = vsum + wt*(en-e_trial)**2
c set weights = 1
c          vsum = vsum + (en-e_trial)**2
  
       enddo
       tme = etime(tme2)
       tot2= tme2(1)
c       print*,'det, jas, tot',det_time,jas_time,tot2-tot1

c       ave = sum/dble(nconfig)
c       var= vsum/dble(nconfig)
       ave = sum/wt_sum
       var= vsum/wt_sum
      
       evar = ssq/dble(nconfig) - ave*ave
c       err = sqrt(evar/(nconfig-1))
       sigma = sqrt(evar)
c       print*,'evar',evar,sigma

       f = var
       neff = wt_sum**2/(nconfig*wt_ssq)
    
       print*,"var = ",var,neff,nf


      end
      
