
       subroutine orbital_init()
          print*,'-- Periodic boundary conditions in orbital'
       end
       subroutine orbital(dist3,centers,rotmat,vp,ob)
          implicit none 
          include 'param.h'
          
          real*8 dist3(ndim)
          real*8 centers(ndim,ncenter)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
          real*8 ob(npair)
          real*8 tv(npair),tv3(ndim,npair)

          integer i,j,ib
          real*8 arg
          real*8 g11,g33,cd,aa,newr(3)
          real*8 gc,fc,fcexp,rm


          gc = 1.d0/vp(nvp_per*npair+id)**2
          call orb_dist(centers,dist3,tv,tv3)
          rm = 0.5d0*box_len


          do i=1,npair
             if (tv(i).lt.rm) then
                ib = id + nvp_per*(i-1)
                g11 = 1.d0/vp(ib)**2
                g33 = 1.d0/vp(ib+1)**2
                cd  = vp(ib+2)
 
                do j=1,ndim
                   newr(j)  = rotmat(j,1,i)*tv3(1,i) + 
     >                        rotmat(j,2,i)*tv3(2,i) + 
     >                        rotmat(j,3,i)*tv3(3,i) 
                enddo
                arg = g11*newr(1)**2 + g11*newr(2)**2 + g33*newr(3)**2
                aa = 1.d0 + cd*tv(i)
                fcexp = exp(-gc*(tv(i)-rm)**2)
                fc = 1.d0-fcexp
                ob(i) = aa*exp(-arg)*fc
             else
                ob(i) = 0.d0
             endif

             
             
          enddo
          
       end

       subroutine orbital_d(dist3,centers,rotmat,vp,ob,ob_d,ob_dd)
          implicit none 
          include 'param.h'
          
          real*8 dist3(ndim)
          real*8 centers(ndim,ncenter)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
          real*8 ob(npair),ob_d(ndim,npair),ob_dd(npair)
          real*8 tv(npair),tv3(ndim,npair)

          integer i,j,k,ib
          real*8 arg
          real*8 g11,g33,cd,aa,newr(3),gam(3)
          real*8 gc,fc,fcexp,rm,fcp,fcpp
          real*8 ga(3),gaga,gagb,grada(3),dot,dela


          gc = 1.d0/vp(nvp_per*npair+id)**2
          call orb_dist(centers,dist3,tv,tv3)
          rm = 0.5d0*box_len

          do i=1,npair
             if (tv(i).lt.rm) then
                ib = id + nvp_per*(i-1)
                g11 = 1.d0/vp(ib)**2
                g33 = 1.d0/vp(ib+1)**2
                cd  = vp(ib+2)
                gam(1) = g11
                gam(2) = g11
                gam(3) = g33

                do j=1,ndim
                   newr(j)  = rotmat(j,1,i)*tv3(1,i) + 
     >                        rotmat(j,2,i)*tv3(2,i) + 
     >                        rotmat(j,3,i)*tv3(3,i) 
                enddo

                arg = g11*newr(1)**2 + g11*newr(2)**2 + g33*newr(3)**2
                aa = 1.d0 + cd*tv(i)
                fcexp = exp(-gc*(tv(i)-rm)**2)
                fc = 1.d0-fcexp

                ob(i) = aa*exp(-arg)*fc


                fcp = 2*gc*(tv(i)-rm)*fcexp
                fcpp = gc*(6.d0-4*gc*(tv(i)-rm)**2 - 4*rm/tv(i))*fcexp



                do j=1,ndim
                   dot = 0.d0
                   do k=1,ndim
                      dot = dot + rotmat(k,j,i)*newr(k)*gam(k)
                   enddo
                   ga(j) = 2*dot 
                   grada(j) = -ga(j)*aa*exp(-arg) + 
     >                      (cd/tv(i))*exp(-arg)*tv3(j,i)
                enddo


                gaga = (2*g11*newr(1))**2 +  (2*g11*newr(2))**2  +
     >                 (2*g33*newr(3))**2 
                gagb = ga(1)*tv3(1,i) + ga(2)*tv3(2,i) + ga(3)*tv3(3,i)

                dela = (-4*g11-2*g33+ gaga)*aa*exp(-arg) +
     >                  exp(-arg)*(2*cd/tv(i)-2*(cd/tv(i))*gagb)

                dot = grada(1)*tv3(1,i) + grada(2)*tv3(2,i) + 
     >                 grada(3)*tv3(3,i)

                ob_dd(i) = dela*fc + aa*exp(-arg)*fcpp + 
     >                   2*fcp*dot/tv(i)

                do j=1,ndim
                  ob_d(j,i) = grada(j)*fc + 
     >                        aa*exp(-arg)*fcp*tv3(j,i)/tv(i)
                enddo



             else
                ob(i) = 0.d0
                ob_dd(i) = 0.d0
                ob_d(1,i) = 0.d0
                ob_d(2,i) = 0.d0
                ob_d(3,i) = 0.d0
             endif
          enddo
          
       end

       subroutine set_centers(npos,centers,rotmat)
          implicit none 
          include 'param.h'
          real*8 npos(ndim,nelec)
          real*8 centers(ndim,ncenter)
          real*8 rotmat(ndim,ndim,npair)

          integer i,j,k
          real*8 tmp(3),td,tmp2(3)

          real*8 cos_theta,cos_phi,sin_theta,sin_phi,res(ndim)

          do i=1,npair
             call box_dist(npos(1,2*i),npos(1,2*i-1),td,tmp)
             do j=1,ndim
                tmp2(j) = npos(j,2*i-1) + .5d0*tmp(j)
             enddo
             call box_move(tmp2,centers(1,i))

             cos_theta = tmp(3)/td
             sin_theta = sqrt(1.d0-cos_theta**2)
             if ( (tmp(1)**2+tmp(2)**2).lt.1.d-9) then
                cos_phi = 0.d0
             else
                cos_phi = tmp(1)/sqrt(tmp(1)**2+tmp(2)**2)
             endif
             sin_phi = sign(sqrt(1.d0-cos_phi**2),tmp(2))

             rotmat(1,1,i) = cos_phi*cos_theta
             rotmat(2,1,i) = -sin_phi
             rotmat(3,1,i) = cos_phi*sin_theta
             rotmat(1,2,i) = sin_phi*cos_theta
             rotmat(2,2,i) = cos_phi
             rotmat(3,2,i) = sin_phi*sin_theta
             rotmat(1,3,i) = -sin_theta
             rotmat(2,3,i) = 0.d0
             rotmat(3,3,i) = cos_theta

             do k=1,3 
                res(k) = 0.d0
                do j=1,3
                   res(k) = res(k) + rotmat(k,j,i)*tmp(j)
                enddo
             enddo
             if ((abs(res(1)).gt.1.0d-14).or.
     >           (abs(res(2)).gt.1.0d-14)) then
                print*,'error, not projecting to z axis,i',i,res(1),
     >                    res(2)
             endif
            
          
          enddo
    
       end

c compute 1st derivative terms (wrt vp's)
       subroutine orbital_vp(np,idx,dist3,center,rotmat,vp,vp_deriv)
          implicit none 
          include 'param.h'
          
          integer np,idx
          real*8 dist3(ndim,np)
          real*8 center(ndim)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
          real*8 vp_deriv(npair,nvp_per)
          real*8 tv(npair),tv3(ndim,npair)
           

          integer i,j,ib
          real*8 arg
          real*8 g11,g33,cd,aa,newr(3)
          real*8 gc,fc,fcexp,rm,ob


          gc = 1.d0/vp(nvp_per*npair+id)**2
          call orb_dist(dist3,center,tv,tv3)
          rm = 0.5d0*box_len
          ib = id + nvp_per*(idx-1)
          g11 = 1.d0/vp(ib)**2
          g33 = 1.d0/vp(ib+1)**2
          cd  = vp(ib+2)

          do i=1,np
             if (tv(i).lt.rm) then
 
                do j=1,ndim
                   newr(j)  = rotmat(j,1,idx)*tv3(1,i) + 
     >                        rotmat(j,2,idx)*tv3(2,i) + 
     >                        rotmat(j,3,idx)*tv3(3,i) 
                enddo
                arg = g11*newr(1)**2 + g11*newr(2)**2 + g33*newr(3)**2
                aa = 1.d0 + cd*tv(i)
                fcexp = exp(-gc*(tv(i)-rm)**2)
                fc = 1.d0-fcexp
                ob = aa*exp(-arg)*fc
                vp_deriv(i,1) = 2.d0*(newr(1)**2*g11**1.5d0 + 
     >                                newr(2)**2*g11**1.5d0)*ob
                vp_deriv(i,2) = 2.d0*newr(3)**2*g33**1.5d0*ob
                vp_deriv(i,3) = tv(i)*exp(-arg)*fc
             else
                vp_deriv(i,1) = 0.d0
                vp_deriv(i,2) = 0.d0
                vp_deriv(i,3) = 0.d0
             endif
             
          enddo
          
       end

c also compute diagonal 2nd derivative term
       subroutine orbital_vp_diag(np,idx,dist3,center,rotmat,vp,
     >                            vp_deriv)
          implicit none 
          include 'param.h'
          
          integer np,idx
          real*8 dist3(ndim,np)
          real*8 center(ndim)
          real*8 rotmat(ndim,ndim,npair),vp(nvp)
          real*8 vp_deriv(npair,nvp_per)
          real*8 tv(npair),tv3(ndim,npair)
           

          integer i,j,ib
          real*8 arg
          real*8 g11,g33,cd,aa,newr(3)
          real*8 gc,fc,fcexp,rm,ob


          gc = 1.d0/vp(nvp_per*npair+id)**2
          call orb_dist(dist3,center,tv,tv3)
          rm = 0.5d0*box_len
          ib = id + nvp_per*(idx-1)
          g11 = 1.d0/vp(ib)**2
          g33 = 1.d0/vp(ib+1)**2
          cd  = vp(ib+2)

          do i=1,np
             if (tv(i).lt.rm) then
 
                do j=1,ndim
                   newr(j)  = rotmat(j,1,idx)*tv3(1,i) + 
     >                        rotmat(j,2,idx)*tv3(2,i) + 
     >                        rotmat(j,3,idx)*tv3(3,i) 
                enddo
                arg = g11*newr(1)**2 + g11*newr(2)**2 + g33*newr(3)**2
                aa = 1.d0 + cd*tv(i)
                fcexp = exp(-gc*(tv(i)-rm)**2)
                fc = 1.d0-fcexp
                ob = aa*exp(-arg)*fc
                vp_deriv(i,1) = 2.d0*(newr(1)**2*g11**1.5d0 + 
     >                                newr(2)**2*g11**1.5d0)*ob
                vp_deriv(i,2) = 2.d0*newr(3)**2*g33**1.5d0*ob
                vp_deriv(i,3) = tv(i)*exp(-arg)*fc
             else
                vp_deriv(i,1) = 0.d0
                vp_deriv(i,2) = 0.d0
                vp_deriv(i,3) = 0.d0
             endif
             
          enddo
          
       end
